@@ -20,8 +20,9 @@
 
 package Lintian::languages::php;
 
-use strict;
+use v5.20;
 use warnings;
+use utf8;
 use autodie;
 
 use Moo;
